# -*- coding: utf-8 -*-
"""

    ファイル名：   Build.py

    処理内容：

        配布形式のビルド

"""

from __future__ import division, print_function, unicode_literals
import sys
import os
import platform
import shutil
from subprocess         import call
from RequiredPackages   import check_tested
from BasicUtils         import clear_dirs_with_retry, rename_with_retry

def build( thisdir, updir ):
    check_tested()

    main_script = 'synthesizer.py'
    main_name   = main_script.replace( '.py', '' )

    bitness = platform.architecture()[0]
    if bitness == '64bit':
        assert( updir.find( 'x64' ) >= 0 )
    elif bitness == '32bit':
        assert( updir.find( 'x86' ) >= 0 )
    else:
        assert( False )

    print( 'Targeted architecture bitness seems to be consistent.' )

    ret = call( 'upx -V > nul', shell=True )
    if ret == 0:
        print( 'UPX is available.' )
        upx_is_avalable = True
    else:
        print( 'UPX is not available.' )
        upx_is_avalable = False

    built_dir = './dist/%s' % ( main_name )
    temp_dirs = [ 'build', 'dist', 'temp' ]
    for dir_ in temp_dirs:
        if os.path.exists( dir_ ):
            shutil.rmtree( dir_ )

    ret = call( 'pyinstaller %s --onedir --windowed --icon=../lib/%s.ico' % ( main_script, main_name ), shell=True )
    assert( ret == 0 )

    assert( os.path.exists( built_dir ) )

    python_install_dir = os.path.dirname( sys.executable )

    print( "Python is installed in '%s'" % ( python_install_dir ) )

    import numpy.core
    numpy_core_dir = os.path.dirname( numpy.core.__file__ )
    print( "numpy.core is in '%s'" % ( numpy_core_dir ) )

    if upx_is_avalable:
        working_dll_dir = 'temp'
        clear_dirs_with_retry( [ working_dll_dir ] )
    else:
        working_dll_dir = numpy_core_dir

    existing_dll_names = []
    dll_names = [ 'libiomp5md.dll', 'mkl_avx2.dll', 'mkl_core.dll', 'mkl_def.dll', 'mkl_intel_thread.dll' ]
    for name in dll_names:
        dll_path = os.path.join( numpy_core_dir, name )
        if os.path.exists( dll_path ):
            existing_dll_names.append( name )
            if upx_is_avalable:
                print( 'Compressing %s with UPX...' % ( name ) )
                working_dll_path = os.path.join( working_dll_dir, name )
                shutil.copy2( dll_path, working_dll_path )
                ret = call( 'upx %s' % ( working_dll_path ) )
                assert( ret == 0 )

    for name in existing_dll_names:
        shutil.copy2( os.path.join( working_dll_dir, name ), built_dir )

    if upx_is_avalable:
        import matplotlib
        matplotlib_dir = os.path.dirname( matplotlib.__file__ )

        badly_compressed_dlls = [
            os.path.join( python_install_dir,   'vcruntime140.dll' ),
            os.path.join( matplotlib_dir,       'msvcp140.dll'     ),
            ]

        for path in badly_compressed_dlls:
            name = os.path.split( path )[-1]
            target_path = os.path.join( built_dir, name )
            upxname = name.replace( '.dll', '-upx.dll' )
            rename_path = os.path.join( built_dir, upxname )
            os.rename( target_path, rename_path )
            shutil.copy2( path, target_path )

    dist_dir = os.path.join( updir, main_name )
    if os.path.exists( dist_dir ):
        shutil.rmtree( dist_dir )

    rename_with_retry( built_dir, dist_dir )

    for dir_ in temp_dirs:
        if os.path.exists( dir_ ):
            shutil.rmtree( dir_ )

    os.remove( main_script.replace( '.py', '.spec' ) )

    if upx_is_avalable:
        with_phrase = ' with UPX compress'
    else:
        with_phrase = ''
    print( 'The distribution directory has been successfully built%s.' % ( with_phrase ) )
