# -*- coding: utf-8 -*-
"""

    ファイル名：   PersistentInfo.py

    処理内容：

"""
from __future__ import unicode_literals

import sys
import os
import pickle
import shutil
if sys.version_info > (3,):
    read_open_mode  = 'rb'
    write_open_mode = 'wb'
else:
    read_open_mode  = 'r'
    write_open_mode = 'w'

from BasicUtils     import home_dir, exe_name
from OurTkinter     import Tk
import OurMessageBox    as MessageBox

def kek_tools_folder():
    return '%s/.KekTools' % ( home_dir() )

def get_pickle_file_path( name ):
    return '%s/%s/%s' % ( kek_tools_folder(), exe_name(), name )

def make_pickle_file_path( name ):
    # パラメータ保存用の pickle ファイル名構築
    kektools_folder_ = kek_tools_folder()
    if not os.path.exists( kektools_folder_ ):
        os.makedirs( kektools_folder_ )
    app_folder   = '%s/%s' % ( kektools_folder_, exe_name() )
    if not os.path.exists( app_folder ):
        os.makedirs( app_folder )

    return '%s/%s' % ( app_folder, name )

class PersistentInfo():
    def __init__( self, filename, defaults={} ):

        self.pickle_file = make_pickle_file_path( filename )

        # パラメータのロード
        if os.path.exists( self.pickle_file ) and os.path.getsize( self.pickle_file ):
            try:
                pf = open( self.pickle_file, read_open_mode )
                self.dictionary = pickle.load( pf )
                pf.close()
            except:
                pf.close()
                pickle_dir = os.path.dirname( self.pickle_file )
                MessageBox.showwarning( 'title', 
                    'Maybe "%s" does not conform to the expected-format. Your previous setting information in "%s" will be lost.'
                    % ( self.pickle_file, pickle_dir ) )
                shutil.rmtree( pickle_dir )
                self.dictionary = {}
        else:
            self.dictionary = {}

        for k in defaults.keys():
            if self.dictionary.get(k) == None:
                self.dictionary[k] = defaults[k]

    def set_dictionary( self, dictionary ):
        self.dictionary = dictionary

    def get_dictionary( self ):
        return self.dictionary

    def save( self ):
        pf = open( self.pickle_file, write_open_mode )
        pickle.dump( self.dictionary, pf )
        pf.close()

